package path

import (
	"buildme"
	"buildme/utils"
	"errors"
	"os"
	"path/filepath"

	"dev.hexasoftware.com/hxs/prettylog"
)

var (
	log = prettylog.New("PathFetcher")
)

func init() {
	buildme.RegisterFetcher("path", &Path{})
}

type Path struct{}

func (p *Path) Fetch(fpath string) (*buildme.Project, error) {

	fpath, err := filepath.Abs(fpath)
	if err != nil {
		return nil, err
	}

	s, err := os.Stat(fpath)
	if err != nil {
		return nil, err
	}
	if !s.IsDir() {
		return nil, errors.New("Path should be a directory")
	}
	log.Println("Building path:", fpath)
	projName := filepath.Base(fpath)

	//tmpTarReader, err := utils.Tar(fpath, nil) // in memory
	tarFile, err := utils.TempTar(fpath, buildme.TempDir(), "tarbuildme.")
	if err != nil {
		return nil, err
	}
	tarFile.Close()
	log.Println("Tar file:", tarFile.Name())
	proj := buildme.NewProject(projName)
	proj.SetTarFile(tarFile.Name(), true)

	return proj, nil

}
